require 'colorize'
require "aws_ssh/files"
require "aws_ssh/terminal_formats"
require "aws_ssh/version"
require 'fileutils'
require "aws_ssh/runner"

module AWS_SSH
  class Setup
    attr_accessor :profile

    # all vars are required
    def env
      return ( ENV['AWS_ACCESS_KEY_ID'] && ENV['AWS_SECRET_ACCESS_KEY'] && ENV['AWS_REGION'] && ENV['SHELL'] && ENV['HOME'])
    end

    def shell
      shell = ENV['SHELL']
      last = shell.rindex("/")+1
      len = shell.length-1
      return shell[last..len]
    end

    def self.shells(home, shell)
      ["#{home}/.#{shell}rc", "#{home}/.#{shell}_profile", "#{home}/.profile"]
    end

    def profile
      home = ENV['HOME']
      shell = self.shell
      @profile = nil
      # check for automated locations
      if ! shell.nil?
        possibles = AWS_SSH::Setup.shells(home, shell)
        possibles.each {|p| if @profile.nil? && File.exists?(p) then @profile = p end }
      end

    end

    # create the alias file
    def alias
      msg = "# This is an automatically generated file; do not edit.\n# Created at #{Time.now}\n# Created by aws_ssh #{AWS_SSH::VERSION}\n"
      cmd = 'alias ssh="' + AWS_SSH::GENERATE_CMD + ' ; '+AWS_SSH::MERGE_CMD+ '; ssh"'
      filename=ENV['HOME']+"/"+AWS_SSH::ALIAS_FILE
      File.open(filename, 'w') { |file| file.write(msg+cmd) }
      return File.exists?(filename)
    end
    # move any current ssh config file to a base file
    def create_base
      origin = "#{ENV['HOME']}/.ssh/config"
      dest = "#{ENV['HOME']}/.ssh/#{AWS_SSH::BASE_HOSTS_FILE}"
      if File.exists?(origin)
        FileUtils.mv(origin, dest)
      end
    end

    def add_to_profile
      if ! @profile.nil?
        contents = File.open(@profile, 'rb') {|file| file.read}
        # remove any current references and add new one in
        contents = contents.gsub(". ~/#{AWS_SSH::ALIAS_FILE}", "").chomp + "\n" + ". ~/#{AWS_SSH::ALIAS_FILE}"
        File.open(@profile, 'w') { |file| file.write(contents) }
      end
    end

    def backup
      filename = "#{ENV['HOME']}/.ssh/config"
      if File.exists?(filename)
        FileUtils.copy_file(filename, "#{filename}.bk.#{Time.now.to_i}", true)
      end
    end

    def run
      self.backup

      if !self.env
        printf(AWS_SSH::FORMAT_FATAL, "Environment failed", "\u2620")
        return nil
      else
        printf(AWS_SSH::FORMAT_OK, "Environment vars", "\u2713")
      end

      self.profile
      if @profile.nil?
        printf(AWS_SSH::FORMAT_FATAL, "Shell profile failed", "\u2620")
        return nil
      else
        printf(AWS_SSH::FORMAT_OK, "Shell profile found", "\u2713")
      end

      if self.alias
        printf(AWS_SSH::FORMAT_OK, "Alias file created ", "\u2713")
      else
        printf(AWS_SSH::FORMAT_FATAL, "Alias file creation failed", "\u2620")
        return nil
      end

      self.create_base
      printf(AWS_SSH::FORMAT_OK, "Base SSH file created ", "\u2713")
      self.add_to_profile
      printf(AWS_SSH::FORMAT_OK, "Alias added to shell ", "\u2713")

      run = AWS_SSH::Run.new
      run.force
      run.merge
      printf(AWS_SSH::FORMAT_OK, "Completed first run", "\u2713")
      return true
    end

  end
end